/*
    Victron.Arduino-ESP8266
    A:Pim Rutgers
    E:pim@physee.eu

    Code to grab data from the VE.Direct-Protocol on Arduino / ESP8266.
    Tested on NodeMCU v1.0

    The fields of the serial commands are configured in \"config.h\"

	  2021-02-16 Matthieu LAURENT
    Added WiFi communication for ESP8266-01 (webserver).
    All MPPT (75/15 in my case) values displayed.
    Consts uppercased.

    2022-10-27 Matthieu LAURENT
    Added "auto refresh" javascript code (/readvalues.json).
		
		2023-01-10 Matthieu LAURENT
		Added black background-color
		Added right column with max today and yesterday
		Added Load Power (batvoltage*Loadcurrent)

*/


#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include "user_interface.h"
#include "config.h"
#include "VeDirectFrameHandler.h"


// web server
ESP8266WebServer server(80);


IPAddress espip(192, 168, 0, 32);       // set the desired IP Address
IPAddress espgateway(192, 168, 0, 1);   // set gateway to match your network
IPAddress espsubnet(255, 255, 255, 0);  // set subnet mask to match your network
const char* ssid = "your-ssid";
const char* password = "your-password";

VeDirectFrameHandler myve;

// The array that holds the verified values data (with MPPT_VALUES_BYTES size max for each value)
char value[VE_LastKeyword][MPPT_VALUES_BYTES] = { 0 };

// via the USB serial provided by the NodeMCU.
unsigned long data_count = 0;
unsigned long reset_count = 0;
unsigned long refresh_info_count = 0;


const short int PIN_BUILTIN_LED = 2;   // Builtin blue LED

const short int PIN_HIGH_SUN    =  0;  // GPIO0 to activate "high sun" signal (not implemented)
const short int SUN_LEVEL_HIGH  = 10;  // Sun is high (VPV_Value>10V)  à plus de 10V il fait jour
const short int SUN_LEVEL_LOW   =  6;  // Sun is low  (VPV_Value<6V)   à moins de 6V il fait nuit

int VPV_Value = 0;
double PowerVoltage = 0;

// enable ESP.getVcc() reading :
ADC_MODE(ADC_VCC);


////////////////////////////////////////////////////////////////////////////////////////////////////////
void setup() {

  pinMode(PIN_BUILTIN_LED, OUTPUT);  // Initialize the BUILTIN_LED1 pin as an output
  pinMode(PIN_HIGH_SUN, OUTPUT);     // Initialize the PIN_SUN pin as an output

  // Open VE.Direct serial communications and wait for port to open:
  Serial.begin(19200);
  delay(100);
  Serial.flush();

  // remove AP mode only :
  WiFi.mode(WIFI_STA);

  // WiFi configuration
  WiFi.config(espip, espgateway, espsubnet);
  WiFi.begin(ssid, password);

  // Wait for connected state
  while (WiFi.status() != WL_CONNECTED) {
    digitalWrite(PIN_BUILTIN_LED, LOW);
    delay(100);
    digitalWrite(PIN_BUILTIN_LED, HIGH);
    delay(100);
  }

  // set default page for "/" root URL
  server.on("/", handle_index);                 //Handle Index page
  server.on("/raw.html", handle_raw);           //Handle raw page
  server.on("/readvalues.json", handle_readvalues);  //Handle readvalues page (JSON)
  //Start the server
  server.begin();

  reset_count = millis();
  refresh_info_count = millis();
  VPV_Value = 0;
  PowerVoltage = (double)(ESP.getVcc() / 1024.00);
}


////////////////////////////////////////////////////////////////////////////////////////////////////////
void loop() {
  // Receive information on Serial from MPPT
  ReadVEData();

  // Add your own code here to use the data.
  // Make sure to not used delay(X)s of bigger than 50ms

  // clients connections management
  server.handleClient();

  // changer l'état du toutes les minutes :
  if ((millis() - refresh_info_count) > 60000) {
    // lire la valeur VPV si elle existe
    VPV_Value = GetVPVValue();
    // tester les valeurs pour allumer/éteindre
    if (VPV_Value < 6) { digitalWrite(PIN_HIGH_SUN, LOW); }    
    if (VPV_Value > 10) { digitalWrite(PIN_HIGH_SUN, HIGH); }  
    // tension d'alimentation (autour de 3.3V) :
    PowerVoltage = (double)(ESP.getVcc() / 1024.00);
    // initialiser refresh_info_count :
    refresh_info_count = millis();
  }

  // reset tous les jours :
  if ((millis() - reset_count) > 3600000 * 24) {
    //reset_count = millis(); // pas besoin, on redémarre!
    ESP.restart();
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
void ReadVEData() {
  while (Serial.available()) {
    myve.rxData(Serial.read());
    data_count++;
  }
  yield();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
void handle_index() {
  server.send(200, "text/html", MPPTPage());
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
void handle_raw() {
  server.send(200, "text/plain", TextPage());
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
String TextPage() {
  String message = "ESP8266-01s VE.Direct";
  message += "\r\n";
  message += "\r\n";

  message += "Voltage ";
  PowerVoltage = (double)(ESP.getVcc() / 1024.00);
  message += PowerVoltage;
  message += "V\r\n";

  message += "Update #";
  message += myve.frameCount;
  message += "\r\n";
  message += "data_count ";
  message += data_count;
  message += "B\r\n";
  message += "Working since ";
  message += (int)(millis() / 1000);
  message += "s\r\n\r\n";

  for (int i = 0; i < myve.veEnd; i++) {
    message += myve.veName[i];
    message += ": ";
    message += myve.veValue[i];
    message += "\r\n";
  }

  message += "\r\n";
  return message;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
boolean isNumeric(char chStr[]) {
  unsigned int stringLength = strlen(chStr);
  if (stringLength == 0) {
    return false;
  }
  for (unsigned int i = 0; i < stringLength; ++i) {
    if (isDigit(chStr[i])) {
      continue;
    }
    if (chStr[i] == '.') {
      continue;
    }
    return false;
  }
  return true;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
int GetVPVValue() {
  int value = 0;  // default return value
  // remplir le tableau de valeur
  for (int i = 0; i < myve.veEnd; i++) {
    if (strcmp(myve.veName[i], "VPV") == 0) {
      if (isNumeric(myve.veValue[i])) {
        value = (int)atoi(myve.veValue[i]);
        return value;
      }
      break;
    }
  }
  return value;
}


////////////////////////////////////////////////////////////////////////////////////////////////////////
String MPPTPage() {

  String message = "<!DOCTYPE html>\r\n";
  message += "<html>\r\n";
  message += "<head>\r\n";
  message += "<title>ESP8266-VE.Direct</title>\r\n";
  message += "<meta name=\"viewport\" content=\"width=device-width, initial-scale=1\">\r\n";
  message += "<style type=\"text/css\">\r\n text {font-family: Verdana;}\r\n .w{fill: white;}\r\n .value{fill: white; font-size:14px; font-weight:bold; text-anchor:end;}\r\n .g10{fill: #bcdbf6; font-size:10px;}\r\n</style>\r\n";
  message += "</head>\r\n";
  message += "<body style=\"padding: 0px; margin: 0px; background-color: #000;\">\r\n";

  message += "\r\n";
  message += "<svg xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" id=\"solarsvg\" width=\"100%\" viewBox=\"0 0 350 550\">\r\n";
  message += "<rect width=\"100%\" height=\"100%\" fill=\"#397dc6\" />\r\n";   
	message += "<rect x=\"240\" y=\"0\" width=\"110\" height=\"100%\" fill=\"#1d5281\"></rect>\r\n";
	
  // draw separators
  message += "<rect x=\"0\" y=\"60\"  width=\"240\" height=\"130\" fill=\"#2e69a3\"/>\r\n";
  message += "<rect x=\"0\" y=\"58\"  width=\"240\" height=\"2\"   fill=\"#1d5281\"/>\r\n";
  message += "<rect x=\"0\" y=\"190\" width=\"240\" height=\"20\"  fill=\"#4d8aca\"/>\r\n";
  message += "<rect x=\"0\" y=\"290\" width=\"240\" height=\"20\"  fill=\"#4d8aca\"/>\r\n";
  message += "<rect x=\"0\" y=\"420\" width=\"240\" height=\"20\"  fill=\"#4d8aca\"/>\r\n";
  // right separators
  message += "<rect x=\"240\" y=\"58\"  width=\"110\" height=\"2\" fill=\"#000000\"/>\r\n\r\n";

  // draw PV
  message += "<polygon points=\"3,0 12,0 15,13 0,13\" style=\"fill:#bcdbf6;stroke:none;\" transform=\"translate(10,63)\"/>\r\n";
  message += "<polygon points=\"3,0 12,0 15,13 0,13\" style=\"fill:#bcdbf6;stroke:none;\" transform=\"translate(10,220)\"/>\r\n";
  message += "<polygon points=\"3,0 12,0 15,13 0,13\" style=\"fill:#bcdbf6;stroke:none;\" transform=\"translate(10,255)\"/>\r\n";
  // draw battery
  message += "<polygon points=\"0,2 2,2 2,0 4,0 4,2 11,2 11,0 13,0 13,2 15,2 15,13 0,13\" style=\"fill:#bcdbf6;stroke:none;\" transform=\"translate(10,320)\"/>\r\n";
  message += "<polygon points=\"0,2 2,2 2,0 4,0 4,2 11,2 11,0 13,0 13,2 15,2 15,13 0,13\" style=\"fill:#bcdbf6;stroke:none;\" transform=\"translate(10,355)\"/>\r\n";
  message += "<polygon points=\"0,2 2,2 2,0 4,0 4,2 11,2 11,0 13,0 13,2 15,2 15,13 0,13\" style=\"fill:#bcdbf6;stroke:none;\" transform=\"translate(10,390)\"/>\r\n";
  //draw load (lightning)
  message += "<polygon points=\"7,0 3,8 7,8 7,13 11,5 7,5\" style=\"fill:#bcdbf6;stroke:none;\" transform=\"translate(10,450)\"/>\r\n";
  message += "<polygon points=\"7,0 3,8 7,8 7,13 11,5 7,5\" style=\"fill:#bcdbf6;stroke:none;\" transform=\"translate(10,485)\"/>\r\n";
  message += "<polygon points=\"7,0 3,8 7,8 7,13 11,5 7,5\" style=\"fill:#bcdbf6;stroke:none;\" transform=\"translate(10,520)\"/>\r\n\r\n";

  // General (top) informations
  message += "<text id=\"idProductName\" class=\"w\" x=\"120\" y=\"20\" font-size=\"12\" font-weight=\"bold\" text-anchor=\"middle\"></text>\r\n";
  message += "<text id=\"idTracker\" class=\"g10\" x=\"120\" y=\"35\" text-anchor=\"middle\"></text>\r\n";
  message += "<text id=\"idFrameCountAndPowerVoltage\" class=\"g10\" x=\"120\" y=\"50\" text-anchor=\"middle\"></text>\r\n\r\n";

  // Solar (gauge) informations
  message += "<text class=\"g10\" x=\"30\" y=\"73\" >Solar</text>\r\n";
  message += "<circle r=\"60\" cx=\"120\" cy=\"135\" stroke=\"#4d8aca\" stroke-width=\"10\" stroke-dasharray=\"280, 380\" fill=\"none\" transform=\"rotate(137 120 135)\"/>\r\n";
  message += "<g id=\"idGroupSolarGauge\">\r\n";
  message += "<circle r=\"60\" cx=\"120\" cy=\"135\" stroke=\"#ffffff\" stroke-width=\"5\" stroke-dasharray=\"0, 380\" fill=\"none\" transform=\"rotate(137 120 135)\"/>\r\n";
  message += "<text class=\"w\" x=\"145\" y=\"145\"  font-size=\"32\" text-anchor=\"end\" dominant-baseline=\"text-top\"></text>\r\n";
  message += "<text x=\"145\" y=\"145\" fill=\"#bcdbf6\" font-size=\"14\" text-anchor=\"start\" dominant-baseline=\"text-top\"></text>\r\n";
  message += "</g>\r\n\r\n";

  // Solar informations
  message += "<text class=\"g10\" x=\"10\" y=\"203\">Solar</text>\r\n";
  message += "<text class=\"g10\" x=\"30\" y=\"230\">Voltage</text>\r\n";
  message += "<text id=\"idPVvoltage\" class=\"value\" x=\"230\" y=\"230\"></text>\r\n";
  message += "<text class=\"g10\" x=\"30\" y=\"265\">Current</text>\r\n";
  message += "<text id=\"idPVcurrent\" class=\"value\" x=\"230\" y=\"265\"></text>\r\n\r\n";

  // battery information
  message += "<text id=\"idBatvoltageMode\" class=\"g10\" x=\"10\" y=\"303\"></text>\r\n";
  message += "<text class=\"g10\" x=\"30\" y=\"330\" >Voltage</text>\r\n";
  message += "<text id=\"idBatvoltage\" class=\"value\" x=\"230\" y=\"330\"></text>\r\n";
  message += "<text class=\"g10\" x=\"30\" y=\"365\" >Current</text>\r\n";
  message += "<text id=\"idBatcurrent\" class=\"value\" x=\"230\" y=\"365\"></text>\r\n";
  message += "<text class=\"g10\" x=\"30\" y=\"400\" >State</text>\r\n";
  message += "<text id=\"idChargingState\" class=\"value\" x=\"230\" y=\"400\"></text>\r\n\r\n";

  // output informations
  message += "<text class=\"g10\" x=\"10\" y=\"433\">Load output</text>\r\n";
  message += "<text class=\"g10\" x=\"30\" y=\"460\">State</text>\r\n";
  message += "<text id=\"idLoadState\" class=\"value\" x=\"230\" y=\"460\"></text>\r\n";
  message += "<text class=\"g10\" x=\"30\" y=\"495\">Current</text>\r\n";
  message += "<text id=\"idLoadcurrent\" class=\"value\" x=\"230\" y=\"495\"></text>\r\n";
  message += "<text class=\"g10\" x=\"30\" y=\"530\">Power</text>\r\n";
  message += "<text id=\"idLoadpower\" class=\"value\" x=\"230\" y=\"530\"></text>\r\n\r\n";

  message += "<text class=\"g10\" x=\"295\" y=\"73\" text-anchor=\"middle\">Max today</text>\r\n";
  message += "<text id=\"idMaxpowertoday\" class=\"g10\" x=\"295\" y=\"85\" font-weight=\"bold\" text-anchor=\"middle\"></text>\r\n";
  message += "<text class=\"g10\" x=\"295\" y=\"103\" text-anchor=\"middle\">Max yesterday</text>\r\n";
  message += "<text id=\"idMaxpoweryesterday\" class=\"g10\" x=\"295\" y=\"115\" font-weight=\"bold\" text-anchor=\"middle\"></text>\r\n";

  message += "</svg>\r\n\r\n";

  message += "</body>\r\n\r\n";

  // début de la section javascript :
  message += "<script type='text/javascript'>\r\n";
  // pour le premier affichage
  message += "ReloadData();\r\n";
  // definir l'intervale pour les suivants (2s) :
  message += "setInterval(ReloadData, 2000);\r\n";
  message += "function ReloadData() { ReadValues(); }\r\n";

  message += "function ReadValues()\r\n";
  message += "{\r\n";
  message += "  var xmlhttp = new XMLHttpRequest();\r\n";
  message += "  xmlhttp.onreadystatechange = function() {\r\n";
  message += "    if (xmlhttp.readyState == XMLHttpRequest.DONE && xmlhttp.status == 200){\r\n";
  // remplacer toutes les infos :
  message += "      document.getElementById('idProductName').innerHTML = JSON.parse(xmlhttp.responseText).PRODUCTNAME;\r\n";
  message += "      document.getElementById('idTracker').innerHTML = JSON.parse(xmlhttp.responseText).TRACKER;\r\n";
  message += "      document.getElementById('idFrameCountAndPowerVoltage').innerHTML = JSON.parse(xmlhttp.responseText).FRAMECOUNTPOWERVOLTAGE;\r\n";
                    // pour JSON.parse(xmlhttp.responseText).PVPOWER et JSON.parse(xmlhttp.responseText).SVGDASHARRAY :
  message += "      document.getElementById('idGroupSolarGauge').outerHTML = '<g id=\"idGroupSolarGauge\"><circle r=\"60\" cx=\"120\" cy=\"135\" stroke=\"#ffffff\" stroke-width=\"5\" stroke-dasharray=\"' + JSON.parse(xmlhttp.responseText).SVGDASHARRAY + ', 380\" fill=\"none\" transform=\"rotate(137 120 135)\"/><text class=\"w\" x=\"145\" y=\"145\" font-size=\"32\" text-anchor=\"end\" dominant-baseline=\"text-top\">' + JSON.parse(xmlhttp.responseText).PVPOWER + '</text><text x=\"145\" y=\"145\" fill=\"#bcdbf6\" font-size=\"14\" text-anchor=\"start\" dominant-baseline=\"text-top\">w</text></g>';\r\n";
  message += "      document.getElementById('idPVvoltage').innerHTML = JSON.parse(xmlhttp.responseText).PVVOLTAGE + '&nbsp;V';\r\n";
  message += "      document.getElementById('idPVcurrent').innerHTML = JSON.parse(xmlhttp.responseText).PVCURRENT + '&nbsp;A';\r\n";
  message += "      document.getElementById('idBatvoltageMode').innerHTML = 'Battery (' + JSON.parse(xmlhttp.responseText).BATVOLTAGEMODE + '&nbsp;system voltage)';\r\n";
  message += "      document.getElementById('idBatvoltage').innerHTML = JSON.parse(xmlhttp.responseText).BATVOLTAGE + '&nbsp;V' ;\r\n";
  message += "      document.getElementById('idBatcurrent').innerHTML = JSON.parse(xmlhttp.responseText).BATCURRENT + '&nbsp;A';\r\n";
  message += "      document.getElementById('idChargingState').innerHTML = JSON.parse(xmlhttp.responseText).CHARGINGSTATE;\r\n";
  message += "      document.getElementById('idLoadState').innerHTML = JSON.parse(xmlhttp.responseText).LOADSTATE;\r\n";
  message += "      document.getElementById('idLoadcurrent').innerHTML = JSON.parse(xmlhttp.responseText).LOADCURRENT + '&nbsp;A';\r\n";
  message += "      document.getElementById('idLoadpower').innerHTML = JSON.parse(xmlhttp.responseText).LOADPOWER + '&nbsp;W';\r\n";
  message += "      document.getElementById('idMaxpowertoday').innerHTML = JSON.parse(xmlhttp.responseText).MAXPOWERTODAY + '&nbsp;W';\r\n";
  message += "      document.getElementById('idMaxpoweryesterday').innerHTML = JSON.parse(xmlhttp.responseText).MAXPOWERYESTERDAY + '&nbsp;W';\r\n";
 
  message += "    }\r\n";
  message += "  };\r\n";
  message += "  xmlhttp.open('GET', '/readvalues.json', true);\r\n";
  message += "  xmlhttp.send();\r\n";
  message += "}\r\n";
  // fin de la section javascript
  message += "</script>\r\n";

  // fin du HTML
  message += "</html>\r\n";

  return message;
}


///////////////////////////////////////////////////////////////////////////////////////////////////////
void handle_readvalues() {
  // répondre un format JSON pour un traitement des données une fois la page chargées (auto-refresh)
  String ProductName = "";            // identifiant de l'appareil
  String Tracker = "";                // mode de fonctionnement de l'appareil 
  int PVpower = 0;                    // Puissance photovoltaïque
  String SvgDasharray = "0";          // calcul du graphique des watts
  float PVvoltage = 0.0;              // tension photovoltaïque
  float PVcurrent = 0.0;              // courant photovoltaïque
  String BatvoltageMode = "Unknown";  // tension nominale du/des batteries (12, 24, 36, 48)
  float Batvoltage = 0.0;             // tension de batterie  
  float Batcurrent = 0.0;             // courant de batterie
  String ChargingState = "...";       // état de charge
  String LoadState = "...";           // état de la sortie (on/off)
  float Loadcurrent = 0.0;            // courant en sortie
  float Loadpower = 0.0;              // puissance en sortie
	int Maxpowertoday = 0;              // puissance max aujourd'hui (W) (H21)
	int Maxpoweryesterday = 0;          // puissance max hier (W) (H23)

  // reset (default) values :
  for (unsigned int i = VE_FirstKeyword; i < VE_LastKeyword; ++i) {
    value[i][0] = '\0';
  }

  // remplir le tableau de valeur
  for (int i = 0; i < myve.veEnd; i++) {
    for (unsigned int j = VE_FirstKeyword; j < VE_LastKeyword; ++j) {
      if (strcmp(myve.veName[i], keywords[j]) == 0) {
        strcpy(value[j], myve.veValue[i]);
        continue;
      }
    }
  }
  
  // transformer les données :
  for (int i = 0; i < myve.veEnd; i++) {
    if (strcmp(myve.veName[i], "PID") == 0) { ProductName = GetStringProduct(myve.veValue[i]);
    } else if (strcmp(myve.veName[i], "MPPT") == 0) {
      if (isNumeric(myve.veValue[i])) { Tracker = GetStringTracker(atoi(myve.veValue[i])); }
    } else if (strcmp(myve.veName[i], "VPV") == 0) {
      if (isNumeric(myve.veValue[i])) { PVvoltage = (float)atoi(myve.veValue[i]) / 1000; }
    } else if (strcmp(myve.veName[i], "PPV") == 0) {
      if (isNumeric(myve.veValue[i])) {
        PVpower = (int)atoi(myve.veValue[i]);
        // valeur entre 0 et 280 pour 220 Watt de puissance max (en mode 12V)
        // ou     entre 0 et 280 pour 440 Watt de puissance max (en mode 24V) 
        SvgDasharray = String((int)(280 * PVpower / 220)); // /440...
        if (PVvoltage != 0) {
          PVcurrent = (float)PVpower / PVvoltage;
        }
      }
    } else if (strcmp(myve.veName[i], "V") == 0) {
      if (isNumeric(myve.veValue[i])) {
        Batvoltage = (float)atoi(myve.veValue[i]) / 1000;
        BatvoltageMode = GetStringBatteryVoltageMode(Batvoltage);
      }
    } else if (strcmp(myve.veName[i], "I") == 0) {
      if (isNumeric(myve.veValue[i])) { Batcurrent = (float)atoi(myve.veValue[i]) / 1000; }
    } else if (strcmp(myve.veName[i], "CS") == 0) {
      if (isNumeric(myve.veValue[i])) { ChargingState = GetStringState((int)atoi(myve.veValue[i])); }
    } else if (strcmp(myve.veName[i], "LOAD") == 0) {
      LoadState = String(myve.veValue[i]);
    } else if (strcmp(myve.veName[i], "IL") == 0) {
      if (isNumeric(myve.veValue[i])) { 
			  Loadcurrent = (float)atoi(myve.veValue[i]) / 1000; 
				Loadpower = (float)(Loadcurrent * Batvoltage); 
			}
    } else if (strcmp(myve.veName[i], "H21") == 0) {
      if (isNumeric(myve.veValue[i])) { Maxpowertoday = (int)atoi(myve.veValue[i]); }
    } else if (strcmp(myve.veName[i], "H23") == 0) {
      if (isNumeric(myve.veValue[i])) { Maxpoweryesterday = (int)atoi(myve.veValue[i]); }
    }
  }

  // construction des données JSON à renvoyer :
  String json = "{\r\n";
  json += "\"PRODUCTNAME\": \""                    + ProductName + "\",\r\n";
  json += "\"TRACKER\": \""                        + Tracker + "\",\r\n";
  json += "\"DATACOUNT\": \""                      + String(data_count) + "\",\r\n";
  json += "\"FRAMECOUNTPOWERVOLTAGE\": \"Update #" + String(myve.frameCount) + " / " + String(PowerVoltage) + "Vcc\",\r\n"; // variables globales
  json += "\"PVPOWER\": \""                        + String(PVpower) + "\",\r\n";
  json += "\"SVGDASHARRAY\": \""                   + String(SvgDasharray) + "\",\r\n";
  json += "\"PVVOLTAGE\": \""                      + String(PVvoltage) + "\",\r\n";
  json += "\"PVCURRENT\": \""                      + String(PVcurrent) + "\",\r\n";
  json += "\"BATVOLTAGEMODE\": \""                 + BatvoltageMode + "\",\r\n";
  json += "\"BATVOLTAGE\": \""                     + String(Batvoltage) + "\",\r\n";
  json += "\"BATCURRENT\": \""                     + String(Batcurrent) + "\",\r\n";
  json += "\"CHARGINGSTATE\": \""                  + ChargingState + "\",\r\n";
  json += "\"LOADSTATE\": \""                      + LoadState + "\",\r\n";
  json += "\"LOADCURRENT\": \""                    + String(Loadcurrent) + "\",\r\n";  
  json += "\"LOADPOWER\": \""                      + String(Loadpower) + "\",\r\n";  
  json += "\"MAXPOWERTODAY\": \""                  + String(Maxpowertoday) + "\",\r\n";  
  json += "\"MAXPOWERYESTERDAY\": \""              + String(Maxpoweryesterday) + "\"\r\n";  // sans la virgule
  json += "}\r\n";

  // envoyer la réponse :
  server.send(200, "application/json", json);
}
